<?php
/**
 * Events Calendar Search plugin for Joomla 1.5.x
 *
 * @version     $Id: eventsearch.php 969 2008-02-16 11:24:45Z geraint $
 * @package     Events
 * @subpackage  Mambot Events Calendar
 * @copyright   Copyright (C) 2006-2007 JEvents Project Group
 * @copyright   Copyright (C) 2000 - 2003 Eric Lamette, Dave McDonnell
 * @licence     http://www.gnu.org/copyleft/gpl.html
 * @link        http://joomlacode.org/gf/project/jevents
 */

/** ensure this file is being included by a parent file */
defined( '_JEXEC' ) or die( 'Restricted access' );

// setup for all required function and classes
$file = JPATH_SITE . '/components/com_jevents/mod.defines.php';
if (file_exists($file) ) {
	include_once($file);
	include_once(JEV_LIBS."/modfunctions.php");

} else {
	die ("JEvents Calendar\n<br />This module needs the JEvents component");
}

// Import library dependencies
jimport('joomla.event.plugin');

$mainframe->registerEvent( 'onSearchAreas', 'plgSearchEventsSearchAreas' );
/**
 * @return array An array of search areas
 */
function &plgSearchEventsSearchAreas() {
	static $areas = array(
	'events' => 'JEvents'
	);
	return $areas;
}


class plgSearchEventsearch extends JPlugin {

	/**
	 * Constructor
	 *
	 * For php4 compatability we must not use the __constructor as a constructor for plugins
	 * because func_get_args ( void ) returns a copy of all passed arguments NOT references.
	 * This causes problems with cross-referencing necessary for the observer design pattern.
	 *
	 * @param 	object $subject The object to observe
	 * @param 	array  $config  An array that holds the plugin configuration
	 * @since 1.5
	 */

	function plgSearchEventsearch( &$subject )
	{
		parent::__construct( $subject );

		// load plugin parameters
		$this->_plugin = & JPluginHelper::getPlugin( 'search', 'eventsearch' );
		$this->_params = new JParameter( $this->_plugin->params );
	}

	/**
	* Search method
	*
	* The sql must return the following fields that are used in a common display
	* routine: href, title, section, created, text, browsernav
	* @param string Target search string
	* @param string matching option, exact|any|all
	* @param string ordering option, newest|oldest|popular|alpha|category
	*/
	function onSearch( $text, $phrase='', $ordering='' ) {

		$db	=& JFactory::getDBO();
		$user =& JFactory::getUser();


		$limit = $this->_params->def( 'search_limit', 50 );
		$dateformat = $this->_params->def( 'date_format', "%d %B %Y");

		$limit 		= "\n LIMIT $limit";

		$text = trim( $text );
		if ($text == '') {
			return array();
		}

		$search_ical_attributes  = array('det.summary', 'det.description', 'det.location', 'det.contact', 'det.extra_info');

		$wheres = array();
		$wheres_ical = array();
		switch ($phrase) {
			case 'exact':
				// ical
				$wheres2 = array();
				foreach ($search_ical_attributes as $search_item) {
					$wheres2[] = "LOWER($search_item) LIKE '%$text%'";
				}
				$where_ical = '(' . implode( ') OR (', $wheres2 ) . ')';
				break;
			case 'all':
			case 'any':
			default:
				$words = explode( ' ', $text );

				// ical
				$wheres = array();
				foreach ($words as $word) {
					$wheres2 = array();
					foreach ($search_ical_attributes as $search_item) {
						$wheres2[] = "LOWER($search_item) LIKE '%$word%'";
					}
					$wheres[] = implode( ' OR ', $wheres2 );
				}
				$where_ical = '(' . implode( ($phrase == 'all' ? ') AND (' : ') OR ('), $wheres ) . ')';

				break;
		}

		$morder = '';
		$morder_ical = '';
		switch ($ordering) {
			case 'oldest':
				$order = 'a.created ASC';
				$order_ical = 'det.created ASC';
				break;

			case 'popular':
				$order = 'a.hits DESC';
				$order_ical = 'det.created ASC'; // no hit field available
				break;

			case 'alpha':
				$order = 'a.title ASC';
				$order_ical = 'det.summary ASC';
				break;

			case 'category':
				$order = 'b.title ASC, a.title ASC';
				$morder = 'a.title ASC';
				$order_ical = 'b.title ASC, det.summary ASC';
				$morder_ical = 'det.summary ASC';
				break;

			case 'newest':
			default:
				$order = 'a.created DESC';
				$order_ical = 'det.created DESC';
				break;
		}

		$eventstitle=JText::_("Events Calendar");
		// Now Search Icals
		$display2 = array();
		foreach ($search_ical_attributes as $search_ical_attribute) {
			$display2[] = "$search_ical_attribute";
		}
		$display = 'CONCAT('. implode(", ' ', ", $display2) . ')';
		$query = "SELECT det.summary as title,"
		. "\n det.created as created,"
		. "\n $display as text,"
		. "\n CONCAT('$eventstitle','/',title) AS section,"
		. "\n CONCAT('index.php?option=com_jevents&task=icalrepeat.detail&evid=',min(rpt.rp_id)) AS href,"
		. "\n '2' AS browsernav ,"
		. "\n rpt.startrepeat "
		. "\n FROM (#__jevents_vevent as ev)"
		. "\n INNER JOIN #__categories AS b ON b.id = ev.catid"
		. "\n LEFT  JOIN #__jevents_repetition as rpt ON rpt.eventid = ev.ev_id"
		. "\n LEFT  JOIN #__jevents_vevdetail as det ON det.evdet_id = rpt.eventdetail_id"
		. "\n LEFT  JOIN #__jevents_icsfile as icsf ON icsf.ics_id = ev.icsid"
		. "\n WHERE ($where_ical)"
		. "\n AND icsf.state=1"
		. "\n AND icsf.access <= ".$user->gid
		. "\n AND ev.state=1"
		. "\n AND ev.access<=$user->gid"
		. "\n AND b.access<=$user->gid"
		. "\n AND b.published = '1'"
		. "\n GROUP BY det.evdet_id"
		. "\n ORDER BY " . ($morder_ical ? $morder_ical : $order_ical)
		.	$limit
		;

		$db->setQuery( $query );
		$list_ical = $db->loadObjectList();

		jimport('joomla.utilities.date');
		if ($list_ical){
			foreach ($list_ical as $index=>$item) {
				$startdate = new JDate(strtotime($item->startrepeat));
				$item->title .= " (" . $startdate->toFormat($dateformat) . ")";
				$list_ical[$index] = $item;
			}
		}
		return $list_ical;
	}
}